@@ -44,14 +44,16 @@
 
 __author__ = 'api.jscudder (Jeffrey Scudder)'
 
-
 try:
   from xml.etree import cElementTree as ElementTree
 except ImportError:
   try:
     import cElementTree as ElementTree
   except ImportError:
-    from elementtree import ElementTree
+    try:
+      from xml.etree import ElementTree
+    except ImportError:
+      from elementtree import ElementTree
 
 
 # XML namespaces which are often used in Atom entities.
@@ -60,8 +62,14 @@ ELEMENT_TEMPLATE = '{http://www.w3.org/2005/Atom}%s'
 APP_NAMESPACE = 'http://purl.org/atom/app#'
 APP_TEMPLATE = '{http://purl.org/atom/app#}%s'
 
+# This encoding is used for converting strings before translating the XML
+# into an object.
+XML_STRING_ENCODING = 'utf-8'
+# The desired string encoding for object members.
+MEMBER_STRING_ENCODING = 'utf-8'
 
-def CreateClassFromXMLString(target_class, xml_string):
+
+def CreateClassFromXMLString(target_class, xml_string, string_encoding=None):
   """Creates an instance of the target class from the string contents.
   
   Args:
@@ -71,12 +79,19 @@ def CreateClassFromXMLString(target_class, xml_string):
     xml_string: str A string which contains valid XML. The root element
         of the XML string should match the tag and namespace of the desired
         class.
+    string_encoding: str The character encoding which the xml_string should
+        be converted to before it is interpreted and translated into 
+        objects. The default is None in which case the string encoding
+        is not changed.
 
   Returns:
     An instance of the target class with members assigned according to the
     contents of the XML - or None if the root XML tag and namespace did not
     match those of the target class.
   """
+  encoding = string_encoding or XML_STRING_ENCODING
+  if encoding and isinstance(xml_string, unicode):
+    xml_string = xml_string.encode(encoding)
   tree = ElementTree.fromstring(xml_string)
   return _CreateClassFromElementTree(target_class, tree)
 
@@ -130,22 +145,32 @@ class ExtensionContainer(object):
       self._ConvertElementTreeToMember(child)
     for attribute, value in tree.attrib.iteritems():
       self._ConvertElementAttributeToMember(attribute, value)
-    self.text = tree.text
+    # Encode the text string according to the desired encoding type. (UTF-8)
+    if tree.text:
+      self.text = tree.text.encode(MEMBER_STRING_ENCODING)
     
   def _ConvertElementTreeToMember(self, child_tree, current_class=None):
     self.extension_elements.append(_ExtensionElementFromElementTree(
         child_tree))
 
   def _ConvertElementAttributeToMember(self, attribute, value):
-    self.extension_attributes[attribute] = value
+    # Encode the attribute value's string with the desired type Default UTF-8
+    if value:
+      self.extension_attributes[attribute] = value.encode(
+          MEMBER_STRING_ENCODING)
 
   # One method to create an ElementTree from an object
   def _AddMembersToElementTree(self, tree):
     for child in self.extension_elements:
       child._BecomeChildElement(tree)
     for attribute, value in self.extension_attributes.iteritems():
-      tree.attrib[attribute] = value
-    tree.text = self.text
+      if value:
+        # Decode the value from the desired encoding (default UTF-8).
+        tree.attrib[attribute] = value.decode(MEMBER_STRING_ENCODING)
+    if self.text and not isinstance(self.text, unicode):
+      tree.text = self.text.decode(MEMBER_STRING_ENCODING)
+    else:
+      tree.text = self.text 
 
   def FindExtensions(self, tag=None, namespace=None):
     """Searches extension elements for child nodes with the desired name.
@@ -222,9 +247,13 @@ class AtomBase(ExtensionContainer):
       # Find the member of this class which corresponds to the XML attribute
       # (lookup in current_class._attributes) and set this member to the
       # desired value (using self.__dict__).
-      setattr(self, self.__class__._attributes[attribute], value)
+      if value:
+        # Encode the string to capture non-ascii characters (default UTF-8)
+        setattr(self, self.__class__._attributes[attribute], 
+                value.encode(MEMBER_STRING_ENCODING))
     else:
-      ExtensionContainer._ConvertElementAttributeToMember(self, attribute, value)
+      ExtensionContainer._ConvertElementAttributeToMember(self, attribute, 
+          value)
 
   # Three methods to create an ElementTree from an object
   def _AddMembersToElementTree(self, tree):
@@ -246,7 +275,7 @@ class AtomBase(ExtensionContainer):
     for xml_attribute, member_name in self.__class__._attributes.iteritems():
       member = getattr(self, member_name)
       if member is not None:
-        tree.attrib[xml_attribute] = member
+        tree.attrib[xml_attribute] = member.decode(MEMBER_STRING_ENCODING)
     # Lastly, call the ExtensionContainers's _AddMembersToElementTree to 
     # convert any extension attributes.
     ExtensionContainer._AddMembersToElementTree(self, tree)
@@ -279,9 +308,9 @@ class AtomBase(ExtensionContainer):
     self._AddMembersToElementTree(new_tree)
     return new_tree
 
-  def ToString(self):
+  def ToString(self, string_encoding='UTF-8'):
     """Converts the Atom object to a string containing XML."""
-    return ElementTree.tostring(self._ToElementTree(), encoding="UTF-8")
+    return ElementTree.tostring(self._ToElementTree(), encoding=string_encoding)
 
   def __str__(self):
     return self.ToString()
@@ -709,8 +738,8 @@ class Content(Text):
   _attributes = Text._attributes.copy()
   _attributes['src'] = 'src'
 
-  def __init__(self, content_type=None, src=None, text=None, extension_elements=None,
-      extension_attributes=None):
+  def __init__(self, content_type=None, src=None, text=None, 
+      extension_elements=None, extension_attributes=None):
     """Constructor for Content
     
     Args:
@@ -1035,8 +1064,8 @@ class FeedEntryParent(AtomBase, LinkFinder):
   _children['{%s}updated' % ATOM_NAMESPACE] = ('updated', Updated)
 
   def __init__(self, author=None, category=None, contributor=None, 
-      atom_id=None, link=None, rights=None, title=None, updated=None, text=None, 
-      extension_elements=None, extension_attributes=None):
+      atom_id=None, link=None, rights=None, title=None, updated=None, 
+      text=None, extension_elements=None, extension_attributes=None):
     self.author = author or []
     self.category = category or []
     self.contributor = contributor or []
@@ -1063,7 +1092,8 @@ class Source(FeedEntryParent):
   _children['{%s}subtitle' % ATOM_NAMESPACE] = ('subtitle', Subtitle)
 
   def __init__(self, author=None, category=None, contributor=None,
-      generator=None, icon=None, atom_id=None, link=None, logo=None, rights=None, subtitle=None, title=None, updated=None, text=None,
+      generator=None, icon=None, atom_id=None, link=None, logo=None, 
+      rights=None, subtitle=None, title=None, updated=None, text=None,
       extension_elements=None, extension_attributes=None):
     """Constructor for Source
 
